@@ -185,6 +185,7 @@ enum {
 struct mac_session {
     SSLContextRef context;
     struct schan_transport *transport;
+    CRITICAL_SECTION cs;
 };
 
 
@@ -428,7 +429,7 @@ static const struct cipher_suite cipher_suites[] = {
 static const struct cipher_suite* get_cipher_suite(SSLCipherSuite cipher_suite)
 {
     int i;
-    for (i = 0; i < sizeof(cipher_suites)/sizeof(cipher_suites[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(cipher_suites); i++)
     {
         if (cipher_suites[i].suite == cipher_suite)
             return &cipher_suites[i];
@@ -738,10 +739,13 @@ BOOL schan_imp_create_session(schan_imp_session *session, schan_credentials *cre
 
     TRACE("(%p)\n", session);
 
-    s = HeapAlloc(GetProcessHeap(), 0, sizeof(*s));
+    s = heap_alloc(sizeof(*s));
     if (!s)
         return FALSE;
 
+    InitializeCriticalSection(&s->cs);
+    s->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": mac_session.cs");
+
     status = SSLNewContext(cred->credential_use == SECPKG_CRED_INBOUND, &s->context);
     if (status != noErr)
     {
@@ -763,7 +767,7 @@ BOOL schan_imp_create_session(schan_imp_session *session, schan_credentials *cre
         goto fail;
     }
 
-    for(i=0; i < sizeof(protocol_priority_flags)/sizeof(*protocol_priority_flags); i++) {
+    for(i = 0; i < ARRAY_SIZE(protocol_priority_flags); i++) {
         if(!(protocol_priority_flags[i].enable_flag & supported_protocols))
            continue;
 
@@ -789,7 +793,7 @@ BOOL schan_imp_create_session(schan_imp_session *session, schan_credentials *cre
     return TRUE;
 
 fail:
-    HeapFree(GetProcessHeap(), 0, s);
+    heap_free(s);
     return FALSE;
 }
 
@@ -803,7 +807,8 @@ void schan_imp_dispose_session(schan_imp_session session)
     status = SSLDisposeContext(s->context);
     if (status != noErr)
         ERR("Failed to dispose of session context: %d\n", status);
-    HeapFree(GetProcessHeap(), 0, s);
+    DeleteCriticalSection(&s->cs);
+    heap_free(s);
 }
 
 void schan_imp_set_session_transport(schan_imp_session session,
@@ -1090,7 +1095,9 @@ SECURITY_STATUS schan_imp_send(schan_imp_session session, const void *buffer,
 
     TRACE("(%p/%p, %p, %p/%lu)\n", s, s->context, buffer, length, *length);
 
+    EnterCriticalSection(&s->cs);
     status = SSLWrite(s->context, buffer, *length, length);
+    LeaveCriticalSection(&s->cs);
     if (status == noErr)
         TRACE("Wrote %lu bytes\n", *length);
     else if (status == errSSLWouldBlock)
@@ -1120,7 +1127,9 @@ SECURITY_STATUS schan_imp_recv(schan_imp_session session, void *buffer,
 
     TRACE("(%p/%p, %p, %p/%lu)\n", s, s->context, buffer, length, *length);
 
+    EnterCriticalSection(&s->cs);
     status = SSLRead(s->context, buffer, *length, length);
+    LeaveCriticalSection(&s->cs);
     if (status == noErr || status == errSSLClosedGraceful)
         TRACE("Read %lu bytes\n", *length);
     else if (status == errSSLWouldBlock)
