@@ -23,6 +23,13 @@
 
 #include "config.h"
 
+#include <IOKit/pwr_mgt/IOPMLib.h>
+#define GetCurrentThread Mac_GetCurrentThread
+#define LoadResource Mac_LoadResource
+#include <CoreServices/CoreServices.h>
+#undef GetCurrentThread
+#undef LoadResource
+
 #include "macdrv.h"
 #include "winuser.h"
 #include "wine/unicode.h"
@@ -44,9 +51,6 @@ static CFMutableDictionaryRef win_datas;
 
 static DWORD activate_on_focus_time;
 
-/* CrossOver Hack #15388 */
-int quicken_signin_hack = 0;
-
 
 /***********************************************************************
  *              get_cocoa_window_features
@@ -60,13 +64,6 @@ static void get_cocoa_window_features(struct macdrv_win_data *data,
     if (disable_window_decorations) return;
     if (IsRectEmpty(&data->window_rect)) return;
 
-    /* CrossOver Hack #15388 */
-    if (quicken_signin_hack)
-    {
-        wf->shadow = TRUE;
-        wf->title_bar = TRUE;
-        wf->close_button = (GetWindowLongW(GetAncestor(data->hwnd, GA_ROOT), GWL_STYLE) & WS_SYSMENU) != 0;
-    }
     if ((style & WS_CAPTION) == WS_CAPTION && !(ex_style & WS_EX_LAYERED))
     {
         wf->shadow = TRUE;
@@ -675,7 +672,6 @@ static void create_cocoa_window(struct macdrv_win_data *data)
     struct macdrv_thread_data *thread_data = macdrv_init_thread_data();
     WCHAR text[1024];
     struct macdrv_window_features wf;
-    RECT rect;
     CGRect frame;
     DWORD style, ex_style;
     HRGN win_rgn;
@@ -699,20 +695,13 @@ static void create_cocoa_window(struct macdrv_win_data *data)
 
     get_cocoa_window_features(data, style, ex_style, &wf);
 
-    rect = data->whole_rect;
-    /* CrossOver Hack #15388 */
-    if (quicken_signin_hack)
-    {
-        MapWindowPoints(GetAncestor(data->hwnd, GA_PARENT), 0, (POINT *)&rect, 2);
-        OffsetRect(&rect, 22, 22);
-    }
-    frame = cgrect_from_rect(rect);
+    frame = cgrect_from_rect(data->whole_rect);
     constrain_window_frame(&frame);
     if (frame.size.width < 1 || frame.size.height < 1)
         frame.size.width = frame.size.height = 1;
 
-    TRACE("creating %p window %s whole %s client %s frame %s\n", data->hwnd, wine_dbgstr_rect(&data->window_rect),
-          wine_dbgstr_rect(&data->whole_rect), wine_dbgstr_rect(&data->client_rect), wine_dbgstr_cgrect(frame));
+    TRACE("creating %p window %s whole %s client %s\n", data->hwnd, wine_dbgstr_rect(&data->window_rect),
+          wine_dbgstr_rect(&data->whole_rect), wine_dbgstr_rect(&data->client_rect));
 
     data->cocoa_window = macdrv_create_cocoa_window(&wf, frame, data->hwnd, thread_data->queue);
     if (!data->cocoa_window) goto done;
@@ -721,11 +710,7 @@ static void create_cocoa_window(struct macdrv_win_data *data)
     set_cocoa_window_properties(data);
 
     /* set the window text */
-    if (!InternalGetWindowText(data->hwnd, text, sizeof(text)/sizeof(WCHAR))) text[0] = 0;
-    /* CrossOver Hack #15388 */
-    if (quicken_signin_hack && !text[0] &&
-        !InternalGetWindowText(GetAncestor(data->hwnd, GA_ROOT), text, sizeof(text)/sizeof(WCHAR)))
-        text[0] = 0;
+    if (!InternalGetWindowText(data->hwnd, text, ARRAY_SIZE(text))) text[0] = 0;
     macdrv_set_cocoa_window_title(data->cocoa_window, text, strlenW(text));
 
     /* set the window region */
@@ -827,31 +812,6 @@ static void set_cocoa_view_parent(struct macdrv_win_data *data, HWND parent)
 }
 
 
-/* CrossOver Hack #15388 */
-static BOOL needs_cocoa_window(HWND hwnd, HWND parent)
-{
-    DWORD pid;
-    char class[32];
-
-    if (parent == GetDesktopWindow())
-        return TRUE;
-
-    GetWindowThreadProcessId(parent, &pid);
-    if (pid == GetCurrentProcessId())
-        return FALSE;
-
-    if (!GetClassNameA(hwnd, class, sizeof(class)))
-        return FALSE;
-    if (!strcmp(class, "eo.webbrowser.root"))
-    {
-        quicken_signin_hack = 1;
-        return TRUE;
-    }
-
-    return FALSE;
-}
-
-
 /***********************************************************************
  *              macdrv_create_win_data
  *
@@ -879,7 +839,7 @@ static struct macdrv_win_data *macdrv_create_win_data(HWND hwnd, const RECT *win
     data->whole_rect = data->window_rect = *window_rect;
     data->client_rect = *client_rect;
 
-    if (needs_cocoa_window(hwnd, parent))
+    if (parent == GetDesktopWindow())
     {
         create_cocoa_window(data);
         TRACE("win %p/%p window %s whole %s client %s\n",
@@ -1168,22 +1128,13 @@ static void sync_client_view_position(struct macdrv_win_data *data)
 static void sync_window_position(struct macdrv_win_data *data, UINT swp_flags, const RECT *old_window_rect,
                                  const RECT *old_whole_rect)
 {
-    RECT rect;
-    CGRect frame;
+    CGRect frame = cgrect_from_rect(data->whole_rect);
     BOOL force_z_order = FALSE;
 
     if (data->cocoa_window)
     {
         if (data->minimized) return;
 
-        rect = data->whole_rect;
-        /* CrossOver Hack #15388 */
-        if (quicken_signin_hack)
-        {
-            MapWindowPoints(GetAncestor(data->hwnd, GA_PARENT), 0, (POINT *)&rect, 2);
-            OffsetRect(&rect, 22, 22);
-        }
-        frame = cgrect_from_rect(rect);
         constrain_window_frame(&frame);
         if (frame.size.width < 1 || frame.size.height < 1)
             frame.size.width = frame.size.height = 1;
@@ -1195,7 +1146,6 @@ static void sync_window_position(struct macdrv_win_data *data, UINT swp_flags, c
         BOOL were_equal = (data->cocoa_view == data->client_cocoa_view);
         BOOL now_equal = EqualRect(&data->whole_rect, &data->client_rect);
 
-        frame = cgrect_from_rect(data->whole_rect);
         if (were_equal && !now_equal)
         {
             data->cocoa_view = macdrv_create_view(frame);
@@ -1651,13 +1601,44 @@ BOOL CDECL macdrv_CreateDesktopWindow(HWND hwnd)
 }
 
 
+static WNDPROC desktop_orig_wndproc;
+
+#define WM_WINE_NOTIFY_ACTIVITY WM_USER
+
+static LRESULT CALLBACK desktop_wndproc_wrapper( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
+{
+    switch (msg)
+    {
+    case WM_WINE_NOTIFY_ACTIVITY:
+    {
+        /* This wakes from display sleep, but doesn't affect the screen saver. */
+        static IOPMAssertionID assertion;
+        IOPMAssertionDeclareUserActivity(CFSTR("Wine user input"), kIOPMUserActiveLocal, &assertion);
+
+        /* This prevents the screen saver, but doesn't wake from display sleep. */
+        /* It's deprecated, but there's no better alternative. */
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+        UpdateSystemActivity(UsrActivity);
+#pragma clang diagnostic pop
+        break;
+    }
+    }
+    return desktop_orig_wndproc( hwnd, msg, wp, lp );
+}
+
 /**********************************************************************
  *              CreateWindow   (MACDRV.@)
  */
 BOOL CDECL macdrv_CreateWindow(HWND hwnd)
 {
     if (hwnd == GetDesktopWindow())
+    {
+        desktop_orig_wndproc = (WNDPROC)SetWindowLongPtrW( GetDesktopWindow(),
+            GWLP_WNDPROC, (LONG_PTR)desktop_wndproc_wrapper );
+
         macdrv_init_clipboard();
+    }
     return TRUE;
 }
 
@@ -1745,9 +1726,9 @@ void CDECL macdrv_SetParent(HWND hwnd, HWND parent, HWND old_parent)
     if (parent == old_parent) return;
     if (!(data = get_win_data(hwnd))) return;
 
-    if (!needs_cocoa_window(hwnd, parent)) /* a child window */
+    if (parent != GetDesktopWindow()) /* a child window */
     {
-        if (needs_cocoa_window(hwnd, old_parent))
+        if (old_parent == GetDesktopWindow())
         {
             /* destroy the old Mac window */
             destroy_cocoa_window(data);
@@ -1834,14 +1815,9 @@ void CDECL macdrv_SetWindowStyle(HWND hwnd, INT offset, STYLESTRUCT *style)
 void CDECL macdrv_SetWindowText(HWND hwnd, LPCWSTR text)
 {
     macdrv_window win;
-    WCHAR buf[1024];
 
     TRACE("%p, %s\n", hwnd, debugstr_w(text));
 
-    /* CrossOver Hack #15388 */
-    if (quicken_signin_hack && !text[0] &&
-        InternalGetWindowText(GetAncestor(hwnd, GA_ROOT), buf, sizeof(buf)/sizeof(WCHAR)))
-        text = buf;
     if ((win = macdrv_get_cocoa_window(hwnd, FALSE)))
         macdrv_set_cocoa_window_title(win, text, strlenW(text));
 }
@@ -1943,7 +1919,7 @@ BOOL CDECL macdrv_UpdateLayeredWindow(HWND hwnd, const UPDATELAYEREDWINDOWINFO *
     char buffer[FIELD_OFFSET(BITMAPINFO, bmiColors[256])];
     BITMAPINFO *bmi = (BITMAPINFO *)buffer;
     void *src_bits, *dst_bits;
-    RECT rect;
+    RECT rect, src_rect;
     HDC hdc = 0;
     HBITMAP dib;
     BOOL ret = FALSE;
@@ -2005,11 +1981,13 @@ BOOL CDECL macdrv_UpdateLayeredWindow(HWND hwnd, const UPDATELAYEREDWINDOWINFO *
         surface->funcs->unlock(surface);
         PatBlt(hdc, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, BLACKNESS);
     }
+    src_rect = rect;
+    if (info->pptSrc) OffsetRect( &src_rect, info->pptSrc->x, info->pptSrc->y );
+    DPtoLP( info->hdcSrc, (POINT *)&src_rect, 2 );
+
     if (!(ret = GdiAlphaBlend(hdc, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
-                              info->hdcSrc,
-                              rect.left + (info->pptSrc ? info->pptSrc->x : 0),
-                              rect.top + (info->pptSrc ? info->pptSrc->y : 0),
-                              rect.right - rect.left, rect.bottom - rect.top,
+                              info->hdcSrc, src_rect.left, src_rect.top,
+                              src_rect.right - src_rect.left, src_rect.bottom - src_rect.top,
                               blend)))
         goto done;
 
@@ -2303,9 +2281,6 @@ void macdrv_window_close_requested(HWND hwnd)
 {
     HMENU sysmenu;
 
-    /* CrossOver Hack #15388 */
-    if (quicken_signin_hack)
-        hwnd = GetAncestor(hwnd, GA_ROOT);;
     if (GetClassLongW(hwnd, GCL_STYLE) & CS_NOCLOSE)
     {
         TRACE("not closing win %p class style CS_NOCLOSE\n", hwnd);
@@ -2342,7 +2317,6 @@ void macdrv_window_frame_changed(HWND hwnd, const macdrv_event *event)
     BOOL being_dragged;
 
     if (!hwnd) return;
-    if (quicken_signin_hack) return; /* CrossOver Hack #15388 */
     if (!(data = get_win_data(hwnd))) return;
     if (!data->on_screen || data->minimized)
     {
@@ -2402,35 +2376,30 @@ void macdrv_window_frame_changed(HWND hwnd, const macdrv_event *event)
  */
 void macdrv_window_got_focus(HWND hwnd, const macdrv_event *event)
 {
-    LONG style;
-    HWND top;
+    LONG style = GetWindowLongW(hwnd, GWL_STYLE);
 
     if (!hwnd) return;
 
-    /* CrossOver Hack #15388 */
-    top = GetAncestor(hwnd, GA_ROOT);
-    style = GetWindowLongW(top, GWL_STYLE);
+    TRACE("win %p/%p serial %lu enabled %d visible %d style %08x focus %p active %p fg %p\n",
+          hwnd, event->window, event->window_got_focus.serial, IsWindowEnabled(hwnd),
+          IsWindowVisible(hwnd), style, GetFocus(), GetActiveWindow(), GetForegroundWindow());
 
-    TRACE("win %p/%p top %p serial %lu enabled %d visible %d style %08x focus %p active %p fg %p\n",
-          hwnd, event->window, top, event->window_got_focus.serial, IsWindowEnabled(top),
-          IsWindowVisible(top), style, GetFocus(), GetActiveWindow(), GetForegroundWindow());
-
-    if (can_activate_window(top) && !(style & WS_MINIMIZE))
+    if (can_activate_window(hwnd) && !(style & WS_MINIMIZE))
     {
         /* simulate a mouse click on the caption to find out
          * whether the window wants to be activated */
-        LRESULT ma = SendMessageW(top, WM_MOUSEACTIVATE,
-                                  (WPARAM)top,
+        LRESULT ma = SendMessageW(hwnd, WM_MOUSEACTIVATE,
+                                  (WPARAM)GetAncestor(hwnd, GA_ROOT),
                                   MAKELONG(HTCAPTION,WM_LBUTTONDOWN));
         if (ma != MA_NOACTIVATEANDEAT && ma != MA_NOACTIVATE)
         {
-            TRACE("setting foreground window to %p\n", top);
-            SetForegroundWindow(top);
+            TRACE("setting foreground window to %p\n", hwnd);
+            SetForegroundWindow(hwnd);
             return;
         }
     }
 
-    TRACE("win %p/%p top %p rejecting focus\n", hwnd, event->window, top);
+    TRACE("win %p/%p rejecting focus\n", hwnd, event->window);
     macdrv_window_rejected_focus(event);
 }
 
