@@ -597,6 +597,7 @@ struct thread *create_process( int fd, struct thread *parent_thread, int inherit
     list_init( &process->locks );
     list_init( &process->asyncs );
     list_init( &process->classes );
+    list_init( &process->surfaces );
     list_init( &process->views );
     list_init( &process->dlls );
     list_init( &process->rawinput_devices );
@@ -932,6 +933,7 @@ static void process_killed( struct process *process )
     destroy_process_classes( process );
     free_mapped_views( process );
     free_process_user_handles( process );
+    remove_process_surfaces( process );
     remove_process_locks( process );
     set_process_startup_state( process, STARTUP_ABORTED );
     finish_process_tracing( process );
